% RunSimulation.m (script)
%
% Run simulation of "inverted pendulum with two modules/arms".  
% The standard settings:
%   - random packet drops with a probability of 5%
%   - input peak disturbance on input u1
%
% Random number generators a set to fixed seeds.  To change to random seeds
% modify the corresponding part in ConfigureSimParams.m.
%
% --
% Supplementary material to:
% Sebastian Trimpe, "Stability Analysis of Distributed Event-Based State
% Estimation", IEEE Conference on Decision and Control (CDC), 2014.
%
% Copyright:
% Sebastian Trimpe
% Max-Planck-Institute for Intelligent Systems
% Autonomous Motion Department
% strimpe(at)tuebingen.mpg.de
%

% Revision history
% [21.03.14]   preliminary version, published with initial submission
% [15.09.14]   adjustments/updates for final submission
%     

clear;
disp('Simulation of inverted pendulum example.');

%% Parameters/ Config
% Load plant model
load DiscreteTimeLinearModel.mat; 
Ts = linMod_sim.Ts;     % sampling time

% Default simulation params:
ConfigureSimParams;     % load default parameters

% Controller and observer design
DesignControllerAndObserver;


%% Run simulation
% Data stored in workspace.
sim('PendTwoMod');


%% Extract data from simulation

% Triggering decisions (gamma)
gamma = simout_gamma.Data(:,1:3);       % gamma event-based communication
gamma_ex = simout_gamma.Data(:,4:5);    % gamma extra/sync avg communication

% Time
tt = simout_gamma.Time; 
K = size(gamma,1);  % number of data points

% states and estimates
xx = simout_x.Data;
xxhat1 = simout_xhats.Data(:,1:6);
xxhat2 = simout_xhats.Data(:,6+1:2*6);

% inputs
uu = simout_u.Data(:,[3,4]);


%% Calculations
% Compute moving average for communication rates
N = 100;
z = tf('z',Ts);
MAfilter = tf(ones(1,N),N,Ts)/z^(N-1);     

for i=1:size(gamma,2)
    gamma_filt(:,i) = lsim(MAfilter, gamma(:,i),tt);
end;

for i=1:size(gamma_ex,2)
    gamma_ex_filt(:,i) = lsim(MAfilter, gamma_ex(:,i),tt);
end;

% Time instants of synchronous averaging
ind_syncAvg1 = find(gamma_ex(:,1)==4);
ind_syncAvg2 = find(gamma_ex(:,2)==4);
time_syncAvg = tt(ind_syncAvg1);


%% Figure 1: Communication rates
fig1 = figure;

% Plot
plot(tt,gamma_filt);
hold on;
plot(time_syncAvg,0.0*ones(1,length(time_syncAvg)),'.k','Markersize',7);
%
ylabel('Communication rate');
xlabel('Time (s)');
grid;

disp(' ');
disp(['Matlab Figure ',num2str(fig1),' corresponds to Figure 5 in the paper (sequence from 30 to 60s).']);


%% Plot: Figure 2: States and inputs
fig2 = figure;

% Plot 1: arm angles
subplot(3,1,1);
plot(tt,xx(:,[1,3]));
%
ylabel('Arm angle (rad)');
grid;

% Plot 2: pendulum angle
subplot(3,1,2);
plot(tt,xx(:,[5]));
%
ylabel('Pend. angle (rad)');
grid;

% Plot 3: torques
subplot(3,1,3);
plot(tt,uu);
%
ylabel('Input (rad/s)');
xlabel('Time (s)');
grid;

disp(' ');
disp(['Matlab Figure ',num2str(fig2),' corresponds to Figure 4 in the paper  (sequence from 30 to 60s).']);

